@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014 Andrzej <ndrwrdck@gmail.com>
+ *  Copyright (C) 2014-2017 Andrzej <andrzejr@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -43,6 +43,13 @@ gboolean           pulseaudio_config_get_show_notifications         (PulseaudioC
 guint              pulseaudio_config_get_volume_step                (PulseaudioConfig     *config);
 guint              pulseaudio_config_get_volume_max                 (PulseaudioConfig     *config);
 const gchar       *pulseaudio_config_get_mixer_command              (PulseaudioConfig     *config);
+gchar            **pulseaudio_config_get_mpris_players              (PulseaudioConfig     *config);
+
+gboolean           pulseaudio_config_get_enable_mpris               (PulseaudioConfig     *config);
+void               pulseaudio_config_set_mpris_players              (PulseaudioConfig     *config,
+                                                                     gchar               **players);
+void               pulseaudio_config_add_mpris_player               (PulseaudioConfig     *config,
+                                                                     gchar                *player);
 
 G_END_DECLS
 
