@@ -1,4 +1,6 @@
-/*  Copyright (c) 2014-2017 Andrzej <ndrwrdck@gmail.com>
+/*  Copyright (c) 2014-2017 Andrzej <andrzejr@xfce.org>
+ *                2017      Viktor Odintsev <zakhams@gmail.com>
+ *                2017      Matthieu Mota <matthieumota@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -42,6 +44,7 @@
 #include "pulseaudio-button.h"
 #include "pulseaudio-dialog.h"
 #include "pulseaudio-notify.h"
+#include "pulseaudio-mpris.h"
 
 #ifdef HAVE_IDO
 #include <libido/libido.h>
@@ -54,6 +57,7 @@
 #define PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY  "XF86AudioRaiseVolume"
 #define PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY  "XF86AudioLowerVolume"
 #define PULSEAUDIO_PLUGIN_MUTE_KEY          "XF86AudioMute"
+#define PULSEAUDIO_PLUGIN_MIC_MUTE_KEY      "XF86AudioMicMute"
 #endif
 
 
@@ -76,6 +80,8 @@ static void             pulseaudio_plugin_volume_key_pressed               (cons
                                                                             void                  *user_data);
 static void             pulseaudio_plugin_mute_pressed                     (const char            *keystring,
                                                                             void                  *user_data);
+static void             pulseaudio_plugin_mic_mute_pressed                 (const char            *keystring,
+                                                                            void                  *user_data);
 #endif
 
 struct _PulseaudioPluginClass
@@ -99,6 +105,10 @@ struct _PulseaudioPlugin
 
   /* config dialog builder */
   PulseaudioDialog    *dialog;
+
+  /* mpris */
+  PulseaudioMpris     *mpris;
+  gchar              **players;
 };
 
 
@@ -137,6 +147,8 @@ pulseaudio_plugin_init (PulseaudioPlugin *pulseaudio_plugin)
 #ifdef HAVE_LIBNOTIFY
   pulseaudio_plugin->notify            = NULL;
 #endif
+
+  pulseaudio_plugin->mpris             = NULL;
 }
 
 
@@ -194,10 +206,12 @@ pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
 
   const gchar *auth[] =
     {
-      "Andrzej Radecki <ndrwrdck@gmail.com>",
+      "Andrzej Radecki <andrzejr@xfce.org>",
       "Guido Berhoerster <guido+xfce@berhoerster.name>",
       "Simon Steinbeiss <simon@xfce.org>",
       "Steve Dodier-Lazaro <sidi@xfce.org>",
+      "Viktor Odintsev <zakhams@gmail.com>",
+      "Matthieu Mota <matthieumota@gmail.com>",
       NULL
     };
 
@@ -211,7 +225,7 @@ pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
                          "program-name", PACKAGE_NAME,
                          "comments",     _("Adjust the audio volume of the PulseAudio sound system"),
                          "website",      "http://goodies.xfce.org/projects/panel-plugins/xfce4-pulseaudio-plugin",
-                         "copyright",    _("Copyright \xc2\xa9 2014-2017 Andrzej Radecki\n"),
+                         "copyright",    _("Copyright \xc2\xa9 2014-2017 Andrzej Radecki et al.\n"),
                          "authors",      auth,
                          NULL);
 
@@ -238,13 +252,40 @@ pulseaudio_plugin_size_changed (XfcePanelPlugin *plugin,
 {
   PulseaudioPlugin *pulseaudio_plugin = PULSEAUDIO_PLUGIN (plugin);
   gint              icon_size;
+#if !LIBXFCE4PANEL_CHECK_VERSION (4, 13, 0)
+  GtkStyleContext  *context;
+  GtkBorder         padding;
+  GtkBorder         border;
+  gint              width;
+  gint              xthickness;
+  gint              ythickness;
+#endif
 
   /* The plugin only occupies a single row */
   size /= xfce_panel_plugin_get_nrows (plugin);
-#if LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 13, 0)
   icon_size = xfce_panel_plugin_get_icon_size (plugin);
 #else
-  icon_size = size - 2; // fall-back for older panel versions
+  // fall-back for older panel versions
+  /* Get widget's padding and border to correctly calculate the button's icon size */
+  context = gtk_widget_get_style_context (GTK_WIDGET (pulseaudio_plugin->button));
+  gtk_style_context_get_padding (context, gtk_widget_get_state_flags (GTK_WIDGET (pulseaudio_plugin->button)), &padding);
+  gtk_style_context_get_border (context, gtk_widget_get_state_flags (GTK_WIDGET (pulseaudio_plugin->button)), &border);
+  xthickness = padding.left+padding.right+border.left+border.right;
+  ythickness = padding.top+padding.bottom+border.top+border.bottom;
+
+  width = size - 2 * MAX (xthickness, ythickness);
+  /* Since symbolic icons are usually only provided in 16px we
+   * try to be clever and use size steps */
+
+  if (width <= 21)
+      icon_size = 16;
+  else if (width >=22 && width <= 29)
+      icon_size = 24;
+  else if (width >= 30 && width <= 40)
+      icon_size = 32;
+  else
+      icon_size = width;
 #endif
 
   pulseaudio_button_set_size (pulseaudio_plugin->button, size, icon_size);
@@ -277,7 +318,8 @@ pulseaudio_plugin_bind_keys (PulseaudioPlugin      *pulseaudio_plugin)
 
   success = (keybinder_bind (PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed, pulseaudio_plugin) &&
              keybinder_bind (PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed, pulseaudio_plugin) &&
-             keybinder_bind (PULSEAUDIO_PLUGIN_MUTE_KEY, pulseaudio_plugin_mute_pressed, pulseaudio_plugin));
+             keybinder_bind (PULSEAUDIO_PLUGIN_MUTE_KEY, pulseaudio_plugin_mute_pressed, pulseaudio_plugin) &&
+             keybinder_bind (PULSEAUDIO_PLUGIN_MIC_MUTE_KEY, pulseaudio_plugin_mic_mute_pressed, pulseaudio_plugin));
 
   if (!success)
     g_warning ("Could not have grabbed volume control keys. Is another volume control application (xfce4-volumed) running?");
@@ -295,6 +337,7 @@ pulseaudio_plugin_unbind_keys (PulseaudioPlugin      *pulseaudio_plugin)
   keybinder_unbind (PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed);
   keybinder_unbind (PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed);
   keybinder_unbind (PULSEAUDIO_PLUGIN_MUTE_KEY, pulseaudio_plugin_mute_pressed);
+  keybinder_unbind (PULSEAUDIO_PLUGIN_MIC_MUTE_KEY, pulseaudio_plugin_mic_mute_pressed);
 }
 
 
@@ -325,6 +368,18 @@ pulseaudio_plugin_mute_pressed (const char            *keystring,
 
   pulseaudio_volume_toggle_muted (pulseaudio_plugin->volume);
 }
+
+
+static void
+pulseaudio_plugin_mic_mute_pressed (const char            *keystring,
+                                    void                  *user_data)
+{
+  PulseaudioPlugin *pulseaudio_plugin = PULSEAUDIO_PLUGIN (user_data);
+
+  pulseaudio_debug ("%s pressed", keystring);
+
+  pulseaudio_volume_toggle_muted_mic (pulseaudio_plugin->volume);
+}
 #endif
 
 
@@ -365,10 +420,18 @@ pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
   /* volume controller */
   pulseaudio_plugin->volume = pulseaudio_volume_new (pulseaudio_plugin->config);
 
+  /* initialize mpris support */
+#ifdef HAVE_MPRIS2
+  pulseaudio_plugin->mpris = pulseaudio_mpris_new (pulseaudio_plugin->config);
+  pulseaudio_plugin->players = pulseaudio_mpris_get_available_players (pulseaudio_plugin->mpris);
+#endif
+
   /* instantiate a button box */
   pulseaudio_plugin->button = pulseaudio_button_new (pulseaudio_plugin,
                                                      pulseaudio_plugin->config,
+                                                     pulseaudio_plugin->mpris,
                                                      pulseaudio_plugin->volume);
+
   /* initialize notify wrapper */
 #ifdef HAVE_LIBNOTIFY
   pulseaudio_plugin->notify = pulseaudio_notify_new (pulseaudio_plugin->config,
@@ -378,7 +441,4 @@ pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
 
   gtk_container_add (GTK_CONTAINER (plugin), GTK_WIDGET (pulseaudio_plugin->button));
   gtk_widget_show (GTK_WIDGET (pulseaudio_plugin->button));
-
-
 }
-
