@@ -1,4 +1,5 @@
-/*  Copyright (c) 2014-2015 Andrzej <ndrwrdck@gmail.com>
+/*  Copyright (c) 2014-2017 Andrzej <andrzejr@xfce.org>
+ *                2017      Viktor Odintsev <zakhams@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -76,6 +77,7 @@ struct _PulseaudioVolumeClass
 enum
 {
   VOLUME_CHANGED,
+  VOLUME_MIC_CHANGED,
   LAST_SIGNAL
 };
 
@@ -102,6 +104,13 @@ pulseaudio_volume_class_init (PulseaudioVolumeClass *klass)
                   g_cclosure_marshal_VOID__BOOLEAN,
                   G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 
+  pulseaudio_volume_signals[VOLUME_MIC_CHANGED] =
+    g_signal_new (g_intern_static_string ("volume-mic-changed"),
+                  G_TYPE_FROM_CLASS (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__BOOLEAN,
+                  G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 }
 
 
@@ -112,6 +121,8 @@ pulseaudio_volume_init (PulseaudioVolume *volume)
   volume->connected = FALSE;
   volume->volume = 0.0;
   volume->muted = FALSE;
+  volume->volume_mic = 0.0;
+  volume->muted_mic = FALSE;
   volume->reconnect_timer_id = 0;
 
   volume->pa_mainloop = pa_glib_mainloop_new (NULL);
@@ -171,6 +182,41 @@ pulseaudio_volume_sink_info_cb (pa_context         *context,
 
 
 
+
+static void
+pulseaudio_volume_source_info_cb (pa_context           *context,
+                                  const pa_source_info *i,
+                                  int                   eol,
+                                  void                 *userdata)
+{
+  gboolean  muted_mic;
+  gdouble   vol_mic;
+
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+  pulseaudio_debug ("source info: %s, %s", i->name, i->description);
+
+  muted_mic = !!(i->mute);
+  vol_mic = pulseaudio_volume_v2d (volume, i->volume.values[0]);
+
+  if (volume->muted_mic != muted_mic)
+    {
+      pulseaudio_debug ("Updated Mute Mic: %d -> %d", volume->muted_mic, muted_mic);
+      volume->muted_mic = muted_mic;
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+    }
+
+  if (ABS (volume->volume_mic - vol_mic) > 2e-3)
+    {
+      pulseaudio_debug ("Updated Volume Mic: %04.3f -> %04.3f", volume->volume_mic, vol_mic);
+      volume->volume_mic = vol_mic;
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+    }
+  pulseaudio_debug ("volume mic: %f, muted mic: %d", vol_mic, muted_mic);
+}
+
+
+
 static void
 pulseaudio_volume_server_info_cb (pa_context           *context,
                                   const pa_server_info *i,
@@ -181,14 +227,15 @@ pulseaudio_volume_server_info_cb (pa_context           *context,
 
   pulseaudio_debug ("server: %s@%s, v.%s", i->user_name, i->server_name, i->server_version);
   pa_context_get_sink_info_by_name (context, i->default_sink_name, pulseaudio_volume_sink_info_cb, volume);
+  pa_context_get_source_info_by_name (context, i->default_source_name, pulseaudio_volume_source_info_cb, volume);
 }
 
 
 
 
 static void
-pulseaudio_volume_sink_check (PulseaudioVolume *volume,
-                              pa_context       *context)
+pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
+                                     pa_context       *context)
 {
   g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
 
@@ -209,11 +256,12 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
   switch (t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK)
     {
     case PA_SUBSCRIPTION_EVENT_SINK          :
-      pulseaudio_volume_sink_check (volume, context);
+      pulseaudio_volume_sink_source_check (volume, context);
       pulseaudio_debug ("received sink event");
       break;
 
     case PA_SUBSCRIPTION_EVENT_SOURCE        :
+      pulseaudio_volume_sink_source_check (volume, context);
       pulseaudio_debug ("received source event");
       break;
 
@@ -222,7 +270,7 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
       break;
 
     case PA_SUBSCRIPTION_EVENT_SERVER        :
-      pulseaudio_volume_sink_check (volume, context);
+      pulseaudio_volume_sink_source_check (volume, context);
       pulseaudio_debug ("received server event");
       break;
 
@@ -249,9 +297,10 @@ pulseaudio_volume_context_state_cb (pa_context *context,
 
       pulseaudio_debug ("PulseAudio connection established");
       volume->connected = TRUE;
-      // Check current sink volume manually. PA sink events usually not emitted.
-      pulseaudio_volume_sink_check (volume, context);
+      // Check current sink and source volume manually. PA sink events usually not emitted.
+      pulseaudio_volume_sink_source_check (volume, context);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
       break;
 
     case PA_CONTEXT_FAILED       :
@@ -261,7 +310,10 @@ pulseaudio_volume_context_state_cb (pa_context *context,
       volume->connected = FALSE;
       volume->volume = 0.0;
       volume->muted = FALSE;
+      volume->volume_mic = 0.0;
+      volume->muted_mic = FALSE;
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
       if (volume->reconnect_timer_id == 0)
         volume->reconnect_timer_id = g_timeout_add_seconds
           (5, pulseaudio_volume_reconnect_timeout, volume);
@@ -314,7 +366,6 @@ pulseaudio_volume_connect (PulseaudioVolume *volume)
   err = pa_context_connect (volume->pa_context, NULL, PA_CONTEXT_NOFAIL, NULL);
   if (err < 0)
     g_warning ("pa_context_connect() failed: %s", pa_strerror (err));
-  //g_warning ("pa_context_connect() failed: %s", pa_strerror (pa_context_errno (volume->pa_context)));
 }
 
 
@@ -328,7 +379,7 @@ pulseaudio_volume_reconnect_timeout  (gpointer userdata)
   volume->reconnect_timer_id = 0;
   pulseaudio_volume_connect (volume);
 
-  return FALSE; // stop the timer
+  return FALSE;  // stop the timer
 }
 
 
@@ -369,12 +420,12 @@ static pa_volume_t
 pulseaudio_volume_d2v (PulseaudioVolume *volume,
                        gdouble           vol)
 {
-  gdouble pa_volume;
+  pa_volume_t pa_volume;
 
   g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), PA_VOLUME_MUTED);
 
-  pa_volume = (PA_VOLUME_NORM - PA_VOLUME_MUTED) * vol;
-  pa_volume = (pa_volume_t) pa_volume + PA_VOLUME_MUTED;
+  pa_volume = (pa_volume_t) ((PA_VOLUME_NORM - PA_VOLUME_MUTED) * vol);
+  pa_volume = pa_volume + PA_VOLUME_MUTED;
   /* for safety */
   pa_volume = MIN (MAX (pa_volume, PA_VOLUME_MUTED), PA_VOLUME_MAX);
   return pa_volume;
@@ -406,6 +457,8 @@ pulseaudio_volume_sink_volume_changed (pa_context *context,
     g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, TRUE);
 }
 
+
+
 /* mute setting callbacks */
 /* pa_sink_info_cb_t */
 static void
@@ -450,6 +503,74 @@ pulseaudio_volume_toggle_muted (PulseaudioVolume *volume)
 
 
 
+gboolean
+pulseaudio_volume_get_muted_mic (PulseaudioVolume *volume)
+{
+  g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), FALSE);
+
+  return volume->muted_mic;
+}
+
+
+
+/* final callback for mic volume/mute changes */
+/* pa_context_success_cb_t */
+static void
+pulseaudio_volume_source_volume_changed (pa_context *context,
+                                         int         success,
+                                         void       *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+
+  if (success)
+    g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, TRUE);
+}
+
+
+
+/* mute setting callbacks */
+/* pa_source_info_cb_t */
+static void
+pulseaudio_volume_set_muted_mic_cb1 (pa_context           *context,
+                                     const pa_source_info *i,
+                                     int                   eol,
+                                     void                 *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  pa_context_set_source_mute_by_index (context, i->index, volume->muted_mic, pulseaudio_volume_source_volume_changed, volume);
+}
+
+
+
+void
+pulseaudio_volume_set_muted_mic (PulseaudioVolume *volume,
+                                 gboolean          muted_mic)
+{
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+  g_return_if_fail (volume->pa_context != NULL);
+  g_return_if_fail (pa_context_get_state (volume->pa_context) == PA_CONTEXT_READY);
+
+  if (volume->muted_mic != muted_mic)
+    {
+      volume->muted_mic = muted_mic;
+      pa_context_get_source_info_list (volume->pa_context, pulseaudio_volume_set_muted_mic_cb1, volume);
+    }
+}
+
+
+
+void
+pulseaudio_volume_toggle_muted_mic (PulseaudioVolume *volume)
+{
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+
+  pulseaudio_volume_set_muted_mic (volume, !volume->muted_mic);
+}
+
+
+
 gdouble
 pulseaudio_volume_get_volume (PulseaudioVolume *volume)
 {
@@ -468,13 +589,10 @@ pulseaudio_volume_set_volume_cb2 (pa_context         *context,
                                   int                 eol,
                                   void               *userdata)
 {
-  //char st[PA_CVOLUME_SNPRINT_MAX];
-
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
   if (i == NULL) return;
 
-  //pulseaudio_debug ("*** %s", pa_cvolume_snprint (st, sizeof (st), &i->volume));
-  pa_cvolume_set (&i->volume, 1, pulseaudio_volume_d2v (volume, volume->volume));
+  pa_cvolume_set ((pa_cvolume *)&i->volume, 1, pulseaudio_volume_d2v (volume, volume->volume));
   pa_context_set_sink_volume_by_index (context, i->index, &i->volume, pulseaudio_volume_sink_volume_changed, volume);
 }
 
@@ -493,6 +611,7 @@ pulseaudio_volume_set_volume_cb1 (pa_context           *context,
 }
 
 
+
 void
 pulseaudio_volume_set_volume (PulseaudioVolume *volume,
                               gdouble           vol)
@@ -516,6 +635,70 @@ pulseaudio_volume_set_volume (PulseaudioVolume *volume,
 
 
 
+gdouble
+pulseaudio_volume_get_volume_mic (PulseaudioVolume *volume)
+{
+  g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), 0.0);
+
+  return volume->volume_mic;
+}
+
+
+
+/* volume setting callbacks */
+/* pa_source_info_cb_t */
+static void
+pulseaudio_volume_set_volume_mic_cb2 (pa_context           *context,
+                                      const pa_source_info *i,
+                                      int                   eol,
+                                      void                 *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  pa_cvolume_set ((pa_cvolume *)&i->volume, 1, pulseaudio_volume_d2v (volume, volume->volume_mic));
+  pa_context_set_source_volume_by_index (context, i->index, &i->volume, pulseaudio_volume_source_volume_changed, volume);
+}
+
+
+
+/* pa_server_info_cb_t */
+static void
+pulseaudio_volume_set_volume_mic_cb1 (pa_context           *context,
+                                      const pa_server_info *i,
+                                      void                 *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  pa_context_get_source_info_by_name (context, i->default_source_name, pulseaudio_volume_set_volume_mic_cb2, volume);
+}
+
+
+
+void
+pulseaudio_volume_set_volume_mic (PulseaudioVolume *volume,
+                                  gdouble           vol)
+{
+  gdouble vol_max;
+  gdouble vol_trim;
+
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+  g_return_if_fail (volume->pa_context != NULL);
+  g_return_if_fail (pa_context_get_state (volume->pa_context) == PA_CONTEXT_READY);
+
+  vol_max = pulseaudio_config_get_volume_max (volume->config) / 100.0;
+  vol_trim = MIN (MAX (vol, 0.0), vol_max);
+
+  if (volume->volume_mic != vol_trim)
+    {
+      volume->volume_mic = vol_trim;
+      pa_context_get_server_info (volume->pa_context, pulseaudio_volume_set_volume_mic_cb1, volume);
+    }
+}
+
+
+
 PulseaudioVolume *
 pulseaudio_volume_new (PulseaudioConfig *config)
 {
@@ -528,5 +711,3 @@ pulseaudio_volume_new (PulseaudioConfig *config)
 
   return volume;
 }
-
-
